@@ -16,17 +16,22 @@ echo $WORKSPACE
 sh autogen.sh
 ./configure --with-pgconfig=${PGPATH}/bin/pg_config \
   --with-geosconfig=${GEOS_PATH}/bin/geos-config \
-	--without-protobuf
+  --without-protobuf \
+  --with-library-minor-version
 #make clean
 make
 export err_status=0
 make check RUNTESTFLAGS="-v"
 make install
 make check RUNTESTFLAGS="-v --extension"
-err_status=$?
+if [ "$?" != "0" ]; then
+	err_status=$?
+fi
 
 make garden
-err_status=$?
+if [ "$?" != "0" ]; then
+	err_status=$?
+fi
 
 if [ -d $PGDATA/postmaster.pid ] ; then
 	$PGCTL stop -D $PGDATA -s -m fast
