@@ -26,10 +26,13 @@ make install
 make check RUNTESTFLAGS="-v --extension"
 err_status=$?
 
-
 make garden
 err_status=$?
 
+utils/check_all_upgrades.sh \
+    `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
+err_status=$?
+
 
 if [ -d $PGDATA/postmaster.pid ] ; then
 	$PGCTL stop -D $PGDATA -s -m fast
