@@ -66,6 +66,7 @@ fi
     --with-geosconfig=${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}/bin/geos-config \
     --with-gdalconfig=${PROJECTS}/gdal/rel-${GDAL_VER}w${OS_BUILD}/bin/gdal-config \
     --without-interrupt-tests \
+    --with-library-minor-version \
     --prefix=${PROJECTS}/pg/rel/pg${PG_VER}w${OS_BUILD}
 make clean
 make
@@ -118,7 +119,7 @@ fi
 
 # Test all available upgrades
 # TODO: protect via some variable ?
-if [ "$CHECK_UPGRADE" = "1"]; then
+if [ "$PG_VER" < "14"]; then
   utils/check_all_upgrades.sh \
           `grep '^POSTGIS_' Version.config | cut -d= -f2 | paste -sd '.'`
   if [ "$?" != "0" ]; then
