@@ -1,80 +1,6 @@
-#!/bin/bash
-set -e
-if  [[ "${OVERRIDE}" == '' ]] ; then
-	export GEOS_VER=3.8.1
-	export GDAL_VER=2.4.4
-	export PROJ_VER=5.2.0
-	export SFCGAL_VER=1.3.8
-	export CGAL_VER=5.0
-	export ICON_VER=1.15
-	export ZLIB_VER=1.2.11
-  export PROTOBUF_VER=3.2.0
-	export PROTOBUFC_VER=1.2.1
-	export JSON_VER=0.12
-	export PROJSO=libproj-13.dll
-fi;
+#!/usr/bin/env bash
 
-export PROTOBUF_VER=3.2.0
-export PROTOBUFC_VER=1.2.1
-export JSON_VER=0.12
-export PCRE_VER=8.33
-
-if  [[ "${ICON_VER}" == '' ]] ; then
-  export ICON_VER=1.15
-fi;
-
-echo "ICON_VER ${ICON_VER}"
-
-#set to something even if override is on but not set
-if  [[ "${ZLIB_VER}" == '' ]] ; then
-  export ZLIB_VER=1.2.11
-fi;
-
-
-#set to something even if override is on but not set
-if  [[ "${LIBXML_VER}" == '' ]] ; then
-  export LIBXML_VER=2.7.8
-fi;
-
-#set to something even if override is on but not set
-if  [[ "${CGAL_VER}" == '' ]] ; then
-  export CGAL_VER=4.11
-fi;
-
-#export GDAL_VER=2.4.0
-#export PROJ_VER=5.2.0
-
-echo "ZLIB_VER $ZLIB_VER"
-echo "PROJ_VER $PROJ_VER"
-echo "LIBXML_VER $LIBXML_VER"
-echo "CGAL_VER $CGAL_VER"
-export PROJECTS=/projects
-export MINGPROJECTS=/projects
-export PATHOLD=$PATH
-
-if [ "$OS_BUILD" == "64" ] ; then
-	export MINGHOST=x86_64-w64-mingw32
-else
-	export MINGHOST=i686-w64-mingw32
-fi;
-
-echo PATH AFTER: $PATH
-#export PG_VER=9.2beta2
-export PGWINVER=${PG_VER}edb
-export WORKSPACE=`pwd`
-
-echo PATH BEFORE: $PATH
-
-#export PGHOST=localhost
-#export PGPORT=8442
-export PGUSER=postgres
-#export GEOS_VER=3.4.0dev
-#export GDAL_VER=1.9.1
-export PGPATH=${PROJECTS}/postgresql/rel/pg${PG_VER}w${OS_BUILD}${GCC_TYPE}
-export PGPATHEDB=${PROJECTS}/postgresql/rel/pg${PG_VER}w${OS_BUILD}${GCC_TYPE}edb
-
-export POSTGIS_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}
-export POSTGIS_MICRO_VER=${POSTGIS_MAJOR_VERSION}.${POSTGIS_MINOR_VERSION}.${POSTGIS_MICRO_VERSION}
+. $(dirname $0)/winnie_common.sh
 
 if [ -n "$SOURCE_FOLDER" ]; then
   export POSTGIS_SRC=${PROJECTS}/postgis/$SOURCE_FOLDER
