@@ -18,7 +18,7 @@
  *
  **********************************************************************
  *
- * Copyright 2011-2014 Sandro Santilli <strk@kbt.io>
+ * Copyright 2011-2020 Sandro Santilli <strk@kbt.io>
  * Copyright 2015-2018 Daniel Baston <dbaston@gmail.com>
  * Copyright 2017-2018 Darafei Praliaskouski <me@komzpa.net>
  *
@@ -28,10 +28,10 @@
 #include "liblwgeom.h"
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
+#include "lwrandom.h"
 
 #include <stdarg.h>
 #include <stdlib.h>
-#include <time.h>
 
 LWTIN* lwtin_from_geos(const GEOSGeometry* geom, uint8_t want3d);
 
@@ -104,6 +104,7 @@ static void geos_destroy(size_t count, ...) {
 			GEOSGeom_destroy(g);
 		}
 	}
+	va_end(ap);
 }
 
 /*
@@ -119,9 +120,12 @@ POINTARRAY*
 ptarray_from_GEOSCoordSeq(const GEOSCoordSequence* cs, uint8_t want3d)
 {
 	uint32_t dims = 2;
-	uint32_t size = 0, i;
 	POINTARRAY* pa;
+	uint32_t size = 0;
+#if POSTGIS_GEOS_VERSION < 31000
+	uint32_t i;
 	POINT4D point = { 0.0, 0.0, 0.0, 0.0 };
+#endif
 
 	LWDEBUG(2, "ptarray_fromGEOSCoordSeq called");
 
@@ -142,10 +146,13 @@ ptarray_from_GEOSCoordSeq(const GEOSCoordSequence* cs, uint8_t want3d)
 	LWDEBUGF(4, " output dimensions: %d", dims);
 
 	pa = ptarray_construct((dims == 3), 0, size);
-
+#if POSTGIS_GEOS_VERSION >= 31000
+	GEOSCoordSeq_copyToBuffer(cs, (double*) pa->serialized_pointlist, (dims == 3), 0);
+	return pa;
+#else
 	for (i = 0; i < size; i++)
 	{
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 		GEOSCoordSeq_getX(cs, i, &(point.x));
 		GEOSCoordSeq_getY(cs, i, &(point.y));
 		if (dims >= 3) GEOSCoordSeq_getZ(cs, i, &(point.z));
@@ -159,6 +166,7 @@ ptarray_from_GEOSCoordSeq(const GEOSCoordSequence* cs, uint8_t want3d)
 	}
 
 	return pa;
+#endif
 }
 
 /* Return an LWGEOM from a Geometry */
@@ -250,7 +258,7 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 	uint32_t dims = 2;
 	uint32_t i;
 	int append_points = 0;
-	const POINT3DZ* p3d = NULL;
+	const POINT3D *p3d = NULL;
 	const POINT2D* p2d = NULL;
 	GEOSCoordSeq sq;
 
@@ -270,7 +278,18 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 		}
 	}
 
-	if (!(sq = GEOSCoordSeq_create(pa->npoints + append_points, dims)))
+#if POSTGIS_GEOS_VERSION >= 31000
+	if (append_points == 0) {
+		sq = GEOSCoordSeq_copyFromBuffer((const double*) pa->serialized_pointlist, pa->npoints, FLAGS_GET_Z(pa->flags), FLAGS_GET_M(pa->flags));
+		if (!sq)
+		{
+			lwerror("Error creating GEOS Coordinate Sequence");
+			return NULL;
+		}
+		return sq;
+	}
+#endif
+        if (!(sq = GEOSCoordSeq_create(pa->npoints + append_points, dims)))
 	{
 		lwerror("Error creating GEOS Coordinate Sequence");
 		return NULL;
@@ -280,7 +299,7 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 	{
 		if (dims == 3)
 		{
-			p3d = getPoint3dz_cp(pa, i);
+			p3d = getPoint3d_cp(pa, i);
 			p2d = (const POINT2D*)p3d;
 			LWDEBUGF(4, "Point: %g,%g,%g", p3d->x, p3d->y, p3d->z);
 		}
@@ -290,7 +309,7 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 			LWDEBUGF(4, "Point: %g,%g", p2d->x, p2d->y);
 		}
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 		GEOSCoordSeq_setX(sq, i, p2d->x);
 		GEOSCoordSeq_setY(sq, i, p2d->y);
 		if (dims == 3) GEOSCoordSeq_setZ(sq, i, p3d->z);
@@ -307,14 +326,14 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 	{
 		if (dims == 3)
 		{
-			p3d = getPoint3dz_cp(pa, 0);
+			p3d = getPoint3d_cp(pa, 0);
 			p2d = (const POINT2D*)p3d;
 		}
 		else
 			p2d = getPoint2d_cp(pa, 0);
 		for (i = pa->npoints; i < pa->npoints + append_points; i++)
 		{
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 			GEOSCoordSeq_setX(sq, i, p2d->x);
 			GEOSCoordSeq_setY(sq, i, p2d->y);
 #else
@@ -326,6 +345,7 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 	}
 
 	return sq;
+
 }
 
 static inline GEOSGeometry*
@@ -346,7 +366,7 @@ GBOX2GEOS(const GBOX* box)
 	GEOSCoordSequence* seq = GEOSCoordSeq_create(5, 2);
 	if (!seq) return NULL;
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 	GEOSCoordSeq_setX(seq, 0, box->xmin);
 	GEOSCoordSeq_setY(seq, 0, box->ymin);
 
@@ -407,7 +427,7 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 
 	LWDEBUGF(4, "LWGEOM2GEOS got a %s", lwtype_name(lwgeom->type));
 
-	if (lwgeom_has_arc(lwgeom))
+	if (lwgeom_type_arc(lwgeom))
 	{
 		LWGEOM* lwgeom_stroked = lwgeom_stroke(lwgeom, 32);
 		GEOSGeometry* g = LWGEOM2GEOS(lwgeom_stroked, autofix);
@@ -426,10 +446,10 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 		lwp = (LWPOINT*)lwgeom;
 
 		if (lwgeom_is_empty(lwgeom))
-			g = GEOSGeom_createEmptyPolygon();
+			g = GEOSGeom_createEmptyPoint();
 		else
 		{
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 			sq = ptarray_to_GEOSCoordSeq(lwp->point, 0);
 			g = GEOSGeom_createPoint(sq);
 #else
@@ -493,9 +513,25 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 		}
 		if (!g) return NULL;
 		break;
+
+	case TRIANGLETYPE:
+		if (lwgeom_is_empty(lwgeom))
+			g = GEOSGeom_createEmptyPolygon();
+		else
+		{
+			LWTRIANGLE *lwt = (LWTRIANGLE *)lwgeom;
+			shell = ptarray_to_GEOSLinearRing(lwt->points, autofix);
+			if (!shell)
+				return NULL;
+			g = GEOSGeom_createPolygon(shell, NULL, 0);
+		}
+		if (!g)
+			return NULL;
+		break;
 	case MULTIPOINTTYPE:
 	case MULTILINETYPE:
 	case MULTIPOLYGONTYPE:
+	case TINTYPE:
 	case COLLECTIONTYPE:
 		if (lwgeom->type == MULTIPOINTTYPE)
 			geostype = GEOS_MULTIPOINT;
@@ -516,7 +552,7 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 		{
 			GEOSGeometry* g;
 
-			if (lwgeom_is_empty(lwc->geoms[i])) continue;
+			/* if (lwgeom_is_empty(lwc->geoms[i])) continue; */
 
 			g = LWGEOM2GEOS(lwc->geoms[i], 0);
 			if (!g)
@@ -558,7 +594,7 @@ make_geos_point(double x, double y)
 
 	if (!seq) return NULL;
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 	GEOSCoordSeq_setX(seq, 0, x);
 	GEOSCoordSeq_setY(seq, 0, y);
 #else
@@ -578,7 +614,7 @@ make_geos_segment(double x1, double y1, double x2, double y2)
 
 	if (!seq) return NULL;
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 	GEOSCoordSeq_setX(seq, 0, x1);
 	GEOSCoordSeq_setY(seq, 0, y1);
 	GEOSCoordSeq_setX(seq, 1, x2);
@@ -615,6 +651,7 @@ get_result_srid(size_t count, const char* funcname, ...)
 		if (!g)
 		{
 			lwerror("%s: Geometry is null", funcname);
+			va_end(ap);
 			return SRID_INVALID;
 		}
 		if (i == 0)
@@ -626,10 +663,12 @@ get_result_srid(size_t count, const char* funcname, ...)
 			if (g->srid != srid)
 			{
 				lwerror("%s: Operation on mixed SRID geometries (%d != %d)", funcname, srid, g->srid);
+				va_end(ap);
 				return SRID_INVALID;
 			}
 		}
 	}
+	va_end(ap);
 	return srid;
 }
 
@@ -657,7 +696,13 @@ lwgeom_normalize(const LWGEOM* geom)
 }
 
 LWGEOM*
-lwgeom_intersection(const LWGEOM* geom1, const LWGEOM* geom2)
+lwgeom_intersection(const LWGEOM* g1, const LWGEOM* g2)
+{
+	return lwgeom_intersection_prec(g1, g2, -1.0);
+}
+
+LWGEOM*
+lwgeom_intersection_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom1, geom2);
@@ -679,7 +724,19 @@ lwgeom_intersection(const LWGEOM* geom1, const LWGEOM* geom2)
 	if (!(g1 = LWGEOM2GEOS(geom1, AUTOFIX))) GEOS_FAIL();
 	if (!(g2 = LWGEOM2GEOS(geom2, AUTOFIX))) GEOS_FREE_AND_FAIL(g1);
 
-	g3 = GEOSIntersection(g1, g2);
+	if ( prec >= 0) {
+#if POSTGIS_GEOS_VERSION < 30900
+		lwerror("Fixed-precision intersection requires GEOS-3.9 or higher");
+		GEOS_FREE_AND_FAIL(g1, g2);
+		return NULL;
+#else
+		g3 = GEOSIntersectionPrec(g1, g2, prec);
+#endif
+	}
+	else
+	{
+		g3 = GEOSIntersection(g1, g2);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1);
 	GEOSSetSRID(g3, srid);
@@ -723,6 +780,12 @@ lwgeom_linemerge(const LWGEOM* geom)
 
 LWGEOM*
 lwgeom_unaryunion(const LWGEOM* geom)
+{
+	return lwgeom_unaryunion_prec(geom, -1.0);
+}
+
+LWGEOM*
+lwgeom_unaryunion_prec(const LWGEOM* geom, double prec)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom);
@@ -739,7 +802,19 @@ lwgeom_unaryunion(const LWGEOM* geom)
 
 	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
 
-	g3 = GEOSUnaryUnion(g1);
+	if ( prec >= 0) {
+#if POSTGIS_GEOS_VERSION < 30900
+		lwerror("Fixed-precision union requires GEOS-3.9 or higher");
+		GEOS_FREE_AND_FAIL(g1);
+		return NULL;
+#else
+		g3 = GEOSUnaryUnionPrec(g1, prec);
+#endif
+	}
+	else
+	{
+		g3 = GEOSUnaryUnion(g1);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1);
 	GEOSSetSRID(g3, srid);
@@ -754,6 +829,12 @@ lwgeom_unaryunion(const LWGEOM* geom)
 
 LWGEOM*
 lwgeom_difference(const LWGEOM* geom1, const LWGEOM* geom2)
+{
+	return lwgeom_difference_prec(geom1, geom2, -1.0);
+}
+
+LWGEOM*
+lwgeom_difference_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom1, geom2);
@@ -773,7 +854,19 @@ lwgeom_difference(const LWGEOM* geom1, const LWGEOM* geom2)
 	if (!(g1 = LWGEOM2GEOS(geom1, AUTOFIX))) GEOS_FAIL();
 	if (!(g2 = LWGEOM2GEOS(geom2, AUTOFIX))) GEOS_FREE_AND_FAIL(g1);
 
-	g3 = GEOSDifference(g1, g2);
+	if ( prec >= 0) {
+#if POSTGIS_GEOS_VERSION < 30900
+		lwerror("Fixed-precision difference requires GEOS-3.9 or higher");
+		GEOS_FREE_AND_FAIL(g1, g2);
+		return NULL;
+#else
+		g3 = GEOSDifferencePrec(g1, g2, prec);
+#endif
+	}
+	else
+	{
+		g3 = GEOSDifference(g1, g2);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1, g2);
 	GEOSSetSRID(g3, srid);
@@ -787,6 +880,12 @@ lwgeom_difference(const LWGEOM* geom1, const LWGEOM* geom2)
 
 LWGEOM*
 lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2)
+{
+	return lwgeom_symdifference_prec(geom1, geom2, -1.0);
+}
+
+LWGEOM*
+lwgeom_symdifference_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom1, geom2);
@@ -806,7 +905,19 @@ lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2)
 	if (!(g1 = LWGEOM2GEOS(geom1, AUTOFIX))) GEOS_FAIL();
 	if (!(g2 = LWGEOM2GEOS(geom2, AUTOFIX))) GEOS_FREE_AND_FAIL(g1);
 
-	g3 = GEOSSymDifference(g1, g2);
+	if ( prec >= 0) {
+#if POSTGIS_GEOS_VERSION < 30900
+		lwerror("Fixed-precision difference requires GEOS-3.9 or higher");
+		GEOS_FREE_AND_FAIL(g1, g2);
+		return NULL;
+#else
+		g3 = GEOSSymDifferencePrec(g1, g2, prec);
+#endif
+	}
+	else
+	{
+		g3 = GEOSSymDifference(g1, g2);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1, g2);
 	GEOSSetSRID(g3, srid);
@@ -851,6 +962,42 @@ lwgeom_centroid(const LWGEOM* geom)
 	return result;
 }
 
+LWGEOM*
+lwgeom_reduceprecision(const LWGEOM* geom, double gridSize)
+{
+#if POSTGIS_GEOS_VERSION < 30900
+	lwerror("Precision reduction requires GEOS-3.9 or higher");
+	return NULL;
+#else
+
+	LWGEOM* result;
+	int32_t srid = RESULT_SRID(geom);
+	uint8_t is3d = FLAGS_GET_Z(geom->flags);
+	GEOSGeometry *g1, *g3;
+
+	if (srid == SRID_INVALID) return NULL;
+
+	if (lwgeom_is_empty(geom))
+		return lwgeom_clone_deep(geom);
+
+	initGEOS(lwnotice, lwgeom_geos_error);
+
+	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
+
+	g3 = GEOSGeom_setPrecision(g1, gridSize, 0);
+
+	if (!g3) GEOS_FREE_AND_FAIL(g1);
+	GEOSSetSRID(g3, srid);
+
+	if (!(result = GEOS2LWGEOM(g3, is3d)))
+		GEOS_FREE_AND_FAIL(g1);
+
+	GEOS_FREE(g1, g3);
+
+	return result;
+#endif
+}
+
 LWGEOM *
 lwgeom_pointonsurface(const LWGEOM *geom)
 {
@@ -885,7 +1032,13 @@ lwgeom_pointonsurface(const LWGEOM *geom)
 }
 
 LWGEOM*
-lwgeom_union(const LWGEOM* geom1, const LWGEOM* geom2)
+lwgeom_union(const LWGEOM* g1, const LWGEOM* g2)
+{
+	return lwgeom_union_prec(g1, g2, -1.0);
+}
+
+LWGEOM*
+lwgeom_union_prec(const LWGEOM* geom1, const LWGEOM* geom2, double gridSize)
 {
 	LWGEOM* result;
 	int32_t srid = RESULT_SRID(geom1, geom2);
@@ -905,7 +1058,19 @@ lwgeom_union(const LWGEOM* geom1, const LWGEOM* geom2)
 	if (!(g1 = LWGEOM2GEOS(geom1, AUTOFIX))) GEOS_FAIL();
 	if (!(g2 = LWGEOM2GEOS(geom2, AUTOFIX))) GEOS_FREE_AND_FAIL(g1);
 
-	g3 = GEOSUnion(g1, g2);
+	if ( gridSize >= 0) {
+#if POSTGIS_GEOS_VERSION < 30900
+		lwerror("Fixed-precision union requires GEOS-3.9 or higher");
+		GEOS_FREE_AND_FAIL(g1, g2);
+		return NULL;
+#else
+		g3 = GEOSUnionPrec(g1, g2, gridSize);
+#endif
+	}
+	else
+	{
+		g3 = GEOSUnion(g1, g2);
+	}
 
 	if (!g3) GEOS_FREE_AND_FAIL(g1, g2);
 	GEOSSetSRID(g3, srid);
@@ -951,7 +1116,7 @@ lwgeom_clip_by_rect(const LWGEOM *geom1, double x1, double y1, double x2, double
 }
 
 /* ------------ BuildArea stuff ---------------------------------------------------------------------{ */
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 typedef struct Face_t
 {
 	const GEOSGeometry* geom;
@@ -1216,7 +1381,7 @@ lwgeom_buildarea(const LWGEOM* geom)
 
 	if (!(g1 = LWGEOM2GEOS(geom, AUTOFIX))) GEOS_FAIL();
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 	g3 = LWGEOM_GEOS_buildArea(g1);
 #else
 	g3 = GEOSBuildArea(g1);
@@ -1483,7 +1648,7 @@ lwgeom_offsetcurve(const LWGEOM* geom, double size, int quadsegs, int joinStyle,
 }
 
 LWMPOINT*
-lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
+lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints, int32_t seed)
 {
 	double area, bbox_area, bbox_width, bbox_height;
 	GBOX bbox;
@@ -1499,7 +1664,7 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 	GEOSGeometry* gpt;
 	GEOSCoordSequence* gseq;
 	LWMPOINT* mpt;
-	int srid = lwgeom_get_srid(lwgeom);
+	int32_t srid = lwgeom_get_srid(lwgeom);
 	int done = 0;
 	int* cells;
 	const size_t size = 2 * sizeof(int);
@@ -1537,7 +1702,8 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 	 * http://lin-ear-th-inking.blogspot.ca/2010/05/more-random-points-in-jts.html to try and get a more uniform
 	 * "random" set of points. So we have to figure out how to stick a grid into our box */
 	sample_sqrt = lround(sqrt(sample_npoints));
-	if (sample_sqrt == 0) sample_sqrt = 1;
+	if (sample_sqrt == 0)
+		sample_sqrt = 1;
 
 	/* Calculate the grids we're going to randomize within */
 	if (bbox_width > bbox_height)
@@ -1566,8 +1732,11 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 	/* Get an empty multi-point ready to return */
 	mpt = lwmpoint_construct_empty(srid, 0, 0);
 
-	/* Init random number generator */
-	srand(time(NULL));
+	/* Initiate random number generator.
+	 * Repeatable numbers are generated with seed values >= 1.
+	 * When seed is zero and has not previously been set, it is based on
+	 * Unix time (seconds) and process ID. */
+	lwrandom_set_seed(seed);
 
 	/* Now we fill in an array of cells, and then shuffle that array, */
 	/* so we can visit the cells in random order to avoid visual ugliness */
@@ -1582,14 +1751,13 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 		}
 	}
 
-	/* shuffle */
+	/* Fisher-Yates shuffle */
 	n = sample_height * sample_width;
 	if (n > 1)
 	{
-		for (i = 0; i < n - 1; ++i)
+		for (i = n - 1; i > 0; i--)
 		{
-			size_t rnd = (size_t)rand();
-			size_t j = i + rnd / (RAND_MAX / (n - i) + 1);
+			size_t j = (size_t)(lwrandom_uniform() * (i + 1));
 
 			memcpy(tmp, (char *)cells + j * stride, size);
 			memcpy((char *)cells + j * stride, (char *)cells + i * stride, size);
@@ -1606,12 +1774,12 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 			int contains = 0;
 			double y = bbox.ymin + cells[2 * i] * sample_cell_size;
 			double x = bbox.xmin + cells[2 * i + 1] * sample_cell_size;
-			x += rand() * sample_cell_size / RAND_MAX;
-			y += rand() * sample_cell_size / RAND_MAX;
+			x += lwrandom_uniform() * sample_cell_size;
+			y += lwrandom_uniform() * sample_cell_size;
 			if (x >= bbox.xmax || y >= bbox.ymax) continue;
 
 			gseq = GEOSCoordSeq_create(1, 2);
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 			GEOSCoordSeq_setX(gseq, 0, x);
 			GEOSCoordSeq_setY(gseq, 0, y);
 #else
@@ -1661,7 +1829,7 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints)
 /* Allocate points to sub-geometries by area, then call lwgeom_poly_to_points and bundle up final result in a single
  * multipoint. */
 LWMPOINT*
-lwmpoly_to_points(const LWMPOLY* lwmpoly, uint32_t npoints)
+lwmpoly_to_points(const LWMPOLY* lwmpoly, uint32_t npoints, int32_t seed)
 {
 	const LWGEOM* lwgeom = (LWGEOM*)lwmpoly;
 	double area;
@@ -1683,7 +1851,7 @@ lwmpoly_to_points(const LWMPOLY* lwmpoly, uint32_t npoints)
 		int sub_npoints = lround(npoints * sub_area / area);
 		if (sub_npoints > 0)
 		{
-			LWMPOINT* sub_mpt = lwpoly_to_points(lwmpoly->geoms[i], sub_npoints);
+			LWMPOINT* sub_mpt = lwpoly_to_points(lwmpoly->geoms[i], sub_npoints, seed);
 			if (!mpt)
 				mpt = sub_mpt;
 			else
@@ -1702,14 +1870,14 @@ lwmpoly_to_points(const LWMPOLY* lwmpoly, uint32_t npoints)
 }
 
 LWMPOINT*
-lwgeom_to_points(const LWGEOM* lwgeom, uint32_t npoints)
+lwgeom_to_points(const LWGEOM* lwgeom, uint32_t npoints, int32_t seed)
 {
 	switch (lwgeom_get_type(lwgeom))
 	{
 	case MULTIPOLYGONTYPE:
-		return lwmpoly_to_points((LWMPOLY*)lwgeom, npoints);
+		return lwmpoly_to_points((LWMPOLY*)lwgeom, npoints, seed);
 	case POLYGONTYPE:
-		return lwpoly_to_points((LWPOLY*)lwgeom, npoints);
+		return lwpoly_to_points((LWPOLY*)lwgeom, npoints, seed);
 	default:
 		lwerror("%s: unsupported geometry type '%s'", __func__, lwtype_name(lwgeom_get_type(lwgeom)));
 		return NULL;
@@ -1851,7 +2019,7 @@ lwgeom_get_geos_coordseq_2d(const LWGEOM* g, uint32_t num_points)
 			return NULL;
 		}
 
-#if POSTGIS_GEOS_VERSION < 38
+#if POSTGIS_GEOS_VERSION < 30800
 		if (!GEOSCoordSeq_setX(coords, i, tmp.x) || !GEOSCoordSeq_setY(coords, i, tmp.y))
 #else
 		if (!GEOSCoordSeq_setXY(coords, i, tmp.x, tmp.y))
@@ -1874,7 +2042,7 @@ lwgeom_voronoi_diagram(const LWGEOM* g, const GBOX* env, double tolerance, int o
 	uint32_t num_points = lwgeom_count_vertices(g);
 	LWGEOM* lwgeom_result;
 	char is_3d = LW_FALSE;
-	int srid = lwgeom_get_srid(g);
+	int32_t srid = lwgeom_get_srid(g);
 	GEOSCoordSequence* coords;
 	GEOSGeometry* geos_geom;
 	GEOSGeometry* geos_env = NULL;
