@@ -727,8 +727,6 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 	double vol2 = 1.0;
 	double vol1 = 1.0;
 
-	(void)vol1;
-
 	for ( d = 0 ; d < ndims; d++ )
 	{
 		if ( b1->max[d] <= b2->min[d] || b1->min[d] >= b2->max[d] )
@@ -747,15 +745,15 @@ nd_box_ratio(const ND_BOX *b1, const ND_BOX *b2, int ndims)
 		double width2 = b2->max[d] - b2->min[d];
 		double imin, imax, iwidth;
 
-		vol1 *= width1;
-		vol2 *= width2;
+		vol1 = vol1 * width1;
+		vol2 = vol2 * width2;
 
 		imin = Max(b1->min[d], b2->min[d]);
 		imax = Min(b1->max[d], b2->max[d]);
 		iwidth = imax - imin;
 		iwidth = Max(0.0, iwidth);
 
-		ivol *= iwidth;
+		ivol = ivol * iwidth;
 	}
 
 	if ( vol2 == 0.0 )
@@ -1416,8 +1414,6 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	int stats_slot;                     /* What slot is this data going into? (2D vs ND) */
 	int stats_kind;                     /* And this is what? (2D vs ND) */
 
-	(void)total_sample_volume;
-
 	/* Initialize sum and stddev */
 	nd_box_init(&sum);
 	nd_box_init(&stddev);
@@ -1542,12 +1538,16 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 	/* If there's no useful features, we can't work out stats */
 	if ( ! notnull_cnt )
 	{
+#if POSTGIS_DEBUG_LEVEL > 0
 		Oid relation_oid = stats->attr->attrelid;
 		char *relation_name = get_rel_name(relation_oid);
-		elog(NOTICE,
-		     "PostGIS: Unable to compute statistics for \"%s.%s\": No non-null/empty features",
+		char *namespace = get_namespace_name(get_rel_namespace(relation_oid));
+		elog(DEBUG1,
+		     "PostGIS: Unable to compute statistics for \"%s.%s.%s\": No non-null/empty features",
+		     namespace ? namespace : "(NULL)",
 		     relation_name ? relation_name : "(NULL)",
 		     stats->attr->attname.data);
+#endif /* POSTGIS_DEBUG_LEVEL > 0 */
 		stats->stats_valid = false;
 		return;
 	}
@@ -1767,7 +1767,7 @@ compute_gserialized_stats_mode(VacAttrStats *stats, AnalyzeAttrFetchFunc fetchfu
 		}
 
 		/* Add feature volume (area) to our total */
-		total_sample_volume += tmp_volume;
+		total_sample_volume = total_sample_volume + tmp_volume;
 
 		/*
 		 * Move through all the overlaped histogram cells values and
