@@ -40,6 +40,7 @@
 #include "lwgeom_functions_analytic.h" /* for point_in_polygon */
 #include "lwgeom_geos.h"
 #include "liblwgeom.h"
+#include "liblwgeom_internal.h"
 #include "lwgeom_rtree.h"
 #include "lwgeom_geos_prepared.h"
 #include "lwgeom_accum.h"
@@ -272,7 +273,7 @@ Datum hausdorffdistancedensify(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(ST_FrechetDistance);
 Datum ST_FrechetDistance(PG_FUNCTION_ARGS)
 {
-#if POSTGIS_GEOS_VERSION < 37
+#if POSTGIS_GEOS_VERSION < 30700
 
 	lwpgerror("The GEOS version this PostGIS binary "
 					"was compiled against (%d) doesn't support "
@@ -280,7 +281,7 @@ Datum ST_FrechetDistance(PG_FUNCTION_ARGS)
 					POSTGIS_GEOS_VERSION);
 	PG_RETURN_NULL();
 
-#else /* POSTGIS_GEOS_VERSION >= 37 */
+#else /* POSTGIS_GEOS_VERSION >= 30700 */
 	GSERIALIZED *geom1;
 	GSERIALIZED *geom2;
 	GEOSGeometry *g1;
@@ -328,7 +329,7 @@ Datum ST_FrechetDistance(PG_FUNCTION_ARGS)
 
 	PG_RETURN_FLOAT8(result);
 
-#endif /* POSTGIS_GEOS_VERSION >= 37 */
+#endif /* POSTGIS_GEOS_VERSION >= 30700 */
 }
 
 
@@ -342,7 +343,7 @@ Datum ST_FrechetDistance(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(ST_MaximumInscribedCircle);
 Datum ST_MaximumInscribedCircle(PG_FUNCTION_ARGS)
 {
-#if POSTGIS_GEOS_VERSION < 39
+#if POSTGIS_GEOS_VERSION < 30900
 
 	lwpgerror("The GEOS version this PostGIS binary "
 	          "was compiled against (%d) doesn't support "
@@ -350,7 +351,7 @@ Datum ST_MaximumInscribedCircle(PG_FUNCTION_ARGS)
 	          POSTGIS_GEOS_VERSION);
 	          PG_RETURN_NULL();
 
-#else /* POSTGIS_GEOS_VERSION >= 39 */
+#else /* POSTGIS_GEOS_VERSION >= 30900 */
 	GSERIALIZED* geom;
 	GSERIALIZED* center;
 	GSERIALIZED* nearest;
@@ -459,7 +460,7 @@ Datum ST_MaximumInscribedCircle(PG_FUNCTION_ARGS)
 
 	PG_RETURN_DATUM(result);
 
-#endif /* POSTGIS_GEOS_VERSION >= 39 */
+#endif /* POSTGIS_GEOS_VERSION >= 30900 */
 }
 
 
@@ -517,6 +518,9 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 	if ( count == 0 )
 		PG_RETURN_NULL();
 
+	/* Ok, we really need GEOS now ;) */
+	initGEOS(lwpgnotice, lwgeom_geos_error);
+
 	/* One geom, good geom? Return it */
 	if ( count == 1 && nelems == 1 )
 	{
@@ -524,15 +528,19 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wsign-compare"
 #endif
-		PG_RETURN_POINTER((GSERIALIZED *)(ARR_DATA_PTR(array)));
+		g = POSTGIS2GEOS((GSERIALIZED *)(ARR_DATA_PTR(array)));
 #if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
 #pragma GCC diagnostic pop
 #endif
+		g_union = GEOSUnaryUnion(g);
+		GEOSGeom_destroy(g);
+		if (!g_union) HANDLE_GEOS_ERROR("GEOSUnaryUnion");
+		GEOSSetSRID(g_union, srid);
+		gser_out = GEOS2POSTGIS(g_union, is3d);
+		GEOSGeom_destroy(g_union);
+		PG_RETURN_POINTER(gser_out);
 	}
 
-	/* Ok, we really need GEOS now ;) */
-	initGEOS(lwpgnotice, lwgeom_geos_error);
-
 	/*
 	** Collect the non-empty inputs and stuff them into a GEOS collection
 	*/
@@ -830,79 +838,6 @@ Datum ST_SymDifference(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
-
-PG_FUNCTION_INFO_V1(boundary);
-Datum boundary(PG_FUNCTION_ARGS)
-{
-	GSERIALIZED	*geom1;
-	GEOSGeometry *g1, *g3;
-	GSERIALIZED *result;
-	LWGEOM *lwgeom;
-	int32_t srid;
-
-	geom1 = PG_GETARG_GSERIALIZED_P(0);
-
-	/* Empty.Boundary() == Empty */
-	if ( gserialized_is_empty(geom1) )
-		PG_RETURN_POINTER(geom1);
-
-	srid = gserialized_get_srid(geom1);
-
-	lwgeom = lwgeom_from_gserialized(geom1);
-	if ( ! lwgeom ) {
-		lwpgerror("POSTGIS2GEOS: unable to deserialize input");
-		PG_RETURN_NULL();
-	}
-
-	/* GEOS doesn't do triangle type, so we special case that here */
-	if (lwgeom->type == TRIANGLETYPE)
-	{
-		lwgeom->type = LINETYPE;
-		result = geometry_serialize(lwgeom);
-		lwgeom_free(lwgeom);
-		PG_RETURN_POINTER(result);
-	}
-
-	initGEOS(lwpgnotice, lwgeom_geos_error);
-
-	g1 = LWGEOM2GEOS(lwgeom, 0);
-	lwgeom_free(lwgeom);
-
-	if (!g1)
-		HANDLE_GEOS_ERROR("First argument geometry could not be converted to GEOS");
-
-	g3 = GEOSBoundary(g1);
-
-	if (!g3)
-	{
-		GEOSGeom_destroy(g1);
-		HANDLE_GEOS_ERROR("GEOSBoundary");
-	}
-
-	POSTGIS_DEBUGF(3, "result: %s", GEOSGeomToWKT(g3));
-
-	GEOSSetSRID(g3, srid);
-
-	result = GEOS2POSTGIS(g3, gserialized_has_z(geom1));
-
-	if (!result)
-	{
-		GEOSGeom_destroy(g1);
-		GEOSGeom_destroy(g3);
-		elog(NOTICE,
-		     "GEOS2POSTGIS threw an error (result postgis geometry "
-		     "formation)!");
-		PG_RETURN_NULL(); /* never get here */
-	}
-
-	GEOSGeom_destroy(g1);
-	GEOSGeom_destroy(g3);
-
-	PG_FREE_IF_COPY(geom1, 0);
-
-	PG_RETURN_POINTER(result);
-}
-
 PG_FUNCTION_INFO_V1(convexhull);
 Datum convexhull(PG_FUNCTION_ARGS)
 {
@@ -1642,10 +1577,6 @@ Datum isvalid(PG_FUNCTION_ARGS)
 
 	if ( ! g1 )
 	{
-		/* should we drop the following
-		 * notice now that we have ST_isValidReason ?
-		 */
-		lwpgnotice("%s", lwgeom_geos_errmsg);
 		PG_RETURN_BOOL(false);
 	}
 
