@@ -35,7 +35,7 @@
 #include "lwgeom_pg.h"
 #include "geos_c.h"
 
-#ifdef HAVE_WAGYU
+#ifdef HAVE_LIBPROTOBUF
 #include "lwgeom_wagyu.h"
 #endif
 
@@ -96,8 +96,8 @@ handleInterrupt(int sig)
 
   GEOS_interruptRequest();
 
-#ifdef HAVE_WAGYU
-/* Taking out per #5385 crash */
+#ifdef HAVE_LIBPROTOBUF
+  /* Taking out per #5385 crash */
   //lwgeom_wagyu_interruptRequest();
 #endif
 
