@@ -978,7 +978,7 @@ rt_band_set_pixel(
 	int *converted
 ) {
 	rt_pixtype pixtype = PT_END;
-	unsigned char* data = NULL;
+	uint8_t *data = NULL;
 	uint32_t offset = 0;
 
 	int32_t checkvalint = 0;
@@ -1044,7 +1044,7 @@ rt_band_set_pixel(
 			break;
 		}
 		case PT_8BSI: {
-			data[offset] = rt_util_clamp_to_8BSI(val);
+			data[offset] = (uint8_t)rt_util_clamp_to_8BSI(val);
 			checkvalint = (int8_t) data[offset];
 			break;
 		}
@@ -1205,6 +1205,161 @@ rt_errorstate rt_band_get_pixel_line(
 	return ES_NONE;
 }
 
+/**
+ * Retrieve a point value from the raster using a world coordinate
+ * and selected interpolation.
+ *
+ * @param band : the band to read for values
+ * @param xr : x unrounded raster coordinate
+ * @param yr : y unrounded raster coordinate
+ * @param r_value : return pointer for point value
+ * @param r_nodata : return pointer for if this is a nodata
+ *
+ * @return ES_ERROR on error, otherwise ES_NONE
+ */
+rt_errorstate
+rt_band_get_pixel_resample(
+	rt_band band,
+	double xr, double yr,
+	rt_resample_type resample,
+	double *r_value, int *r_nodata
+)
+{
+	if (resample == RT_BILINEAR) {
+		return rt_band_get_pixel_bilinear(
+			band, xr, yr, r_value, r_nodata
+			);
+	}
+	else if (resample == RT_NEAREST) {
+		return rt_band_get_pixel(
+			band, floor(xr), floor(yr),
+			r_value, r_nodata
+			);
+	}
+	else {
+		rtwarn("Invalid resample type requested %d", resample);
+		return ES_ERROR;
+	}
+
+}
+
+/**
+ * Retrieve a point value from the raster using a world coordinate
+ * and bilinear interpolation.
+ *
+ * @param rast : the raster to read for values
+ * @param bandnum : the band to read for the values
+ * @param xw : x world coordinate in
+ * @param yw : y world coordinate in
+ * @param r_value : return pointer for point value
+ * @param r_nodata : return pointer for if this is a nodata
+ *
+ * @return ES_ERROR on error, otherwise ES_NONE
+ */
+rt_errorstate
+rt_band_get_pixel_bilinear(
+	rt_band band,
+	double xr, double yr,
+	double *r_value, int *r_nodata)
+{
+	double xcenter, ycenter;
+	double values[2][2];
+	double nodatavalue = 0.0;
+	int   nodatas[2][2];
+	int x[2][2];
+	int y[2][2];
+	int xcell, ycell;
+	int xdir, ydir;
+	int i, j;
+	uint16_t width, height;
+
+	/* Cell coordinates */
+	xcell = (int)floor(xr);
+	ycell = (int)floor(yr);
+	xcenter = xcell + 0.5;
+	ycenter = ycell + 0.5;
+
+	/* Raster geometry */
+	width = rt_band_get_width(band);
+	height = rt_band_get_height(band);
+
+	/* Reject out-of-range sample */
+	if(xcell < 0 || ycell < 0 || xcell >= width || ycell >= height) {
+		rtwarn("Attempting to get pixel value with out of range raster coordinates: (%d, %d)", xcell, ycell);
+		return ES_ERROR;
+	}
+
+	/* Quadrant of 2x2 grid the raster coordinate falls in */
+	xdir = xr < xcenter ? 1 : 0;
+	ydir = yr < ycenter ? 1 : 0;
+
+	if (rt_band_get_hasnodata_flag(band) != FALSE) {
+			rt_band_get_nodata(band, &nodatavalue);
+	}
+	else {
+			nodatavalue = 0.0;
+	}
+
+	/* Read the 2x2 values from the band */
+	for (i = 0; i < 2; i++) {
+		for (j = 0; j < 2; j++) {
+			double value = nodatavalue;
+			int nodata = 0;
+			int xij = xcell + (i - xdir);
+			int yij = ycell + (j - ydir);
+
+			if(xij < 0 || yij < 0 || xij >= width || yij >= height) {
+				nodata = 1;
+			}
+			else {
+				rt_errorstate err = rt_band_get_pixel(
+					band, xij, yij,
+					&value, &nodata
+					);
+				if (err != ES_NONE)
+					nodata = 1;
+			}
+			x[i][j] = xij;
+			y[i][j] = yij;
+			values[i][j] = value;
+			nodatas[i][j] = nodata;
+		}
+	}
+
+	/* Point falls in nodata cell, just return nodata */
+	if (nodatas[xdir][ydir]) {
+		*r_value = nodatavalue;
+		*r_nodata = 1;
+		return ES_NONE;
+	}
+
+	/* Normalize raster coordinate to the bottom left */
+	/* so we are working on a unit square */
+	xr = xr - (x[0][0] + 0.5);
+	yr = yr - (y[0][0] + 0.5);
+
+	/* Point is in cell with values, so we take nodata */
+	/* neighbors off the table by matching them to the */
+	/* most controlling cell */
+	for (i = 0; i < 2; i++) {
+		for (j = 0; j < 2; j++) {
+			if (nodatas[i][j])
+				values[i][j] = values[xdir][ydir];
+		}
+	}
+
+	/* Calculate bilinear value */
+	/* https://en.wikipedia.org/wiki/Bilinear_interpolation#Unit_square */
+	*r_nodata = 0;
+	*r_value = values[0][0] * (1-xr) * (1-yr) +
+	           values[1][0] * (1-yr) * xr +
+	           values[0][1] * (1-xr) * yr +
+	           values[1][1] * xr * yr;
+
+	return ES_NONE;
+}
+
+
 /**
  * Get pixel value. If band's isnodata flag is TRUE, value returned
  * will be the band's NODATA value
@@ -1300,7 +1455,7 @@ rt_band_get_pixel(
 			}
 #endif
 		case PT_8BSI: {
-			int8_t val = data[offset];
+			int8_t val = (int8_t)data[offset];
 			*value = val;
 			break;
 		}
@@ -1491,10 +1646,10 @@ uint32_t rt_band_get_nearest_pixel(
 	*npixels = NULL;
 
 	/* maximum extent */
-	max_extent[0] = x - distance[0]; /* min X */
-	max_extent[1] = y - distance[1]; /* min Y */
-	max_extent[2] = x + distance[0]; /* max X */
-	max_extent[3] = y + distance[1]; /* max Y */
+	max_extent[0] = x - (int)distance[0]; /* min X */
+	max_extent[1] = y - (int)distance[1]; /* min Y */
+	max_extent[2] = x + (int)distance[0]; /* max X */
+	max_extent[3] = y + (int)distance[1]; /* max Y */
 	RASTER_DEBUGF(4, "Maximum Extent: (%d, %d, %d, %d)",
 		max_extent[0], max_extent[1], max_extent[2], max_extent[3]);
 
@@ -1504,10 +1659,10 @@ uint32_t rt_band_get_nearest_pixel(
 		_d[0]++;
 		_d[1]++;
 
-		extent[0] = x - _d[0]; /* min x */
-		extent[1] = y - _d[1]; /* min y */
-		extent[2] = x + _d[0]; /* max x */
-		extent[3] = y + _d[1]; /* max y */
+		extent[0] = x - (int)_d[0]; /* min x */
+		extent[1] = y - (int)_d[1]; /* min y */
+		extent[2] = x + (int)_d[0]; /* max x */
+		extent[3] = y + (int)_d[1]; /* max y */
 
 		RASTER_DEBUGF(4, "Processing distances: %d x %d", _d[0], _d[1]);
 		RASTER_DEBUGF(4, "Extent: (%d, %d, %d, %d)",
@@ -1637,6 +1792,8 @@ uint32_t rt_band_get_nearest_pixel(
 	return count;
 }
 
+
+
 /**
  * Search band for pixel(s) with search values
  *
@@ -1755,11 +1912,11 @@ rt_band_check_is_nodata(rt_band band) {
 	int isnodata = 0;
 
 	assert(NULL != band);
+	band->isnodata = FALSE;
 
 	/* Check if band has nodata value */
 	if (!band->hasnodata) {
 		RASTER_DEBUG(3, "Band has no NODATA value");
-		band->isnodata = FALSE;
 		return FALSE;
 	}
 
