@@ -401,6 +401,17 @@ FROM topology.TopoGeo_addLinestring('t4758',
 ORDER BY t;
 SELECT 't4758.end', topology.DropTopology('t4758');
 
+-- See https://trac.osgeo.org/postgis/ticket/2175
+BEGIN;
+SELECT NULL FROM topology.CreateTopology('bug2175');
+SELECT 't2175.1', count(*) from topology.TopoGeo_addLinestring('bug2175',
+  'LINESTRING(10 10,10 0,0 0,0 10,10 10)', 0);
+SELECT 't2175.2', count(*) from topology.TopoGeo_addLinestring('bug2175',
+  'LINESTRING(10 10,10 0,0 0,0 10,10 10)', 0);
+SELECT 't2175.3', count(*) from topology.TopoGeo_addLinestring('bug2175',
+  'LINESTRING(9.99 10,10 0,0 0,0 10,9.99 10)', 0.1);
+ROLLBACK;
+
 -- See https://trac.osgeo.org/postgis/ticket/4941
 SELECT NULL FROM createtopology('b4941');
 SELECT 'b4941', 'outer_ring', count(*) FROM (
@@ -458,6 +469,29 @@ AND NOT ST_Equals(
   );
 SELECT NULL FROM topology.DropTopology('b4941');
 
+-- See https://trac.osgeo.org/postgis/ticket/5081
+SELECT NULL FROM CreateTopology('b5081');
+SELECT 'b5081.0', 'E' || TopoGeo_addLinestring('b5081', $$
+  LINESTRING(0 0, 10 10,10 -10,0 0)
+$$);
+
+SELECT 'b5081.1', count(*) > 1 FROM (
+  SELECT TopoGeo_addLinestring('b5081', $$
+    LINESTRING(
+      -10 0,
+      0 0, 10 10,10 -10,0 0,
+      -10 0
+    )
+  $$)
+) foo;
+SELECT 'b5081', 'dangling_edges', count(*)
+  FROM b5081.edge
+  WHERE left_face = right_face;
+SELECT 'b5081', 'faces', count(*)
+  FROM b5081.face
+  WHERE face_id > 0;
+SELECT NULL FROM topology.DropTopology('b5081');
+
 -- See https://trac.osgeo.org/postgis/ticket/5234
 BEGIN;
 SELECT NULL FROM topology.createtopology('b5234', 0, 0, TRUE);
