@@ -246,13 +246,13 @@ BEGIN
     FOR rec IN
         SELECT e.extname
         FROM
-            pg_catalog.pg_extension e,
-            pg_catalog.pg_depend d
+            pg_extension e,
+            pg_depend d
         WHERE
             d.refclassid = 'pg_catalog.pg_extension'::pg_catalog.regclass AND
             d.refobjid = e.oid AND
-            d.classid = 'pg_catalog.pg_proc'::pg_catalog.regclass AND
-            d.objid = replaced_proc::pg_catalog.oid
+            d.classid = 'pg_proc'::regclass AND
+            d.objid = replaced_proc::oid
     LOOP
         RAISE DEBUG 'Unpackaging ${renamed} from extension %', rec.extname;
         sql := pg_catalog.format('ALTER EXTENSION %I DROP FUNCTION ${renamed}(${args})', rec.extname);
@@ -292,7 +292,7 @@ BEGIN
 EOF
         print "OR version_from_num IN ( ${missing} )" if ($missing);
         print <<"EOF";
-     FROM _postgis_upgrade_info
+     FROM _postgis_upgrade_info()
   THEN
       EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
   END IF;
@@ -375,7 +375,7 @@ BEGIN
     EXECUTE \$postgis_proc_upgrade_parsed_def\$ $pg12_def \$postgis_proc_upgrade_parsed_def\$;
   ELSIF $last_updated > version_from_num OR (
       $last_updated = version_from_num AND version_from_isdev
-    ) FROM _postgis_upgrade_info
+    ) FROM _postgis_upgrade_info()
   THEN
     EXECUTE 'DROP AGGREGATE IF EXISTS $aggsig';
     EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
@@ -411,7 +411,7 @@ EOF
 DO LANGUAGE 'plpgsql'
 \$postgis_proc_upgrade\$
 BEGIN
-  --IF $last_updated > version_from_num FROM _postgis_upgrade_info
+  --IF $last_updated > version_from_num FROM _postgis_upgrade_info()
     --We trust presence of operator rather than version info
     IF NOT EXISTS (
         SELECT o.oprname
@@ -494,7 +494,7 @@ EOF
 DO LANGUAGE 'plpgsql'
 \$postgis_proc_upgrade\$
 BEGIN
-  IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+  IF $last_updated > version_from_num FROM _postgis_upgrade_info() THEN
     EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
   END IF;
 END
@@ -566,7 +566,7 @@ DO LANGUAGE 'plpgsql'
 \$postgis_proc_upgrade\$
 BEGIN
 
-  IF $last_updated > version_from_num FROM _postgis_upgrade_info
+  IF $last_updated > version_from_num FROM _postgis_upgrade_info()
   THEN
     EXECUTE \$postgis_proc_upgrade_parsed_def\$
     $def    \$postgis_proc_upgrade_parsed_def\$;
@@ -579,7 +579,7 @@ EOF
             print <<"EOF";
   $ELSE
     -- Last Updated: ${last_updated}
-    IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+    IF $last_updated > version_from_num FROM _postgis_upgrade_info() THEN
       EXECUTE \$postgis_proc_upgrade_parsed_def\$
         ALTER OPERATOR FAMILY ${opclassname} USING ${opcidx}
           ADD $def;
@@ -613,7 +613,7 @@ DECLARE
 BEGIN
     -- Fetch a list of deprecated functions
 
-    SELECT array_agg(oid::pg_catalog.regprocedure)
+    SELECT array_agg(oid::regprocedure)
     FROM pg_catalog.pg_proc
     WHERE proname = ANY ('${deprecated_names}'::name[])
     INTO deprecated_functions;
@@ -631,8 +631,8 @@ BEGIN
 --                WHEN 'check_option=local' = ANY (c.reloptions) THEN 'WITH LOCAL CHECK OPTION'
 --                ELSE ''
 --            END::text AS check_option
---        FROM pg_catalog.pg_class c
---        LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
+--        FROM pg_class c
+--        LEFT JOIN pg_namespace n ON n.oid = c.relnamespace
 --        WHERE c.relkind = 'v'
 --        AND pg_catalog.pg_get_viewdef(c.oid) ~ 'deprecated_by_postgis'
 --    LOOP
@@ -685,7 +685,7 @@ EOF
 
 close(INPUT);
 
-print "DROP TABLE _postgis_upgrade_info;\n";
+print "DROP FUNCTION _postgis_upgrade_info();\n";
 
 #print "COMMIT;\n";
 
@@ -701,6 +701,8 @@ DECLARE
     new_ver_int int[];
     old_maj text;
     new_maj text;
+    postgis_upgrade_info RECORD;
+    postgis_upgrade_info_func_code TEXT;
 BEGIN
     --
     -- This uses postgis_lib_version() rather then
@@ -768,24 +770,50 @@ BEGIN
     IF old_maj != new_maj THEN
         RAISE EXCEPTION 'Upgrade of MODULE from version % to version % requires a dump/reload. See PostGIS manual for instructions', old_scripts, new_scripts;
     END IF;
+
+    WITH versions AS (
+      SELECT 'NEWVERSION'::text as upgraded,
+      MODULE_scripts_installed() as installed
+    ) SELECT
+      upgraded as scripts_upgraded,
+      installed as scripts_installed,
+      pg_catalog.substring(upgraded, '([0-9]+)\.')::int * 100 +
+      pg_catalog.substring(upgraded, '[0-9]+\.([0-9]+)(\.|$)')::int
+        as version_to_num,
+      pg_catalog.substring(installed, '([0-9]+)\.')::int * 100 +
+      pg_catalog.substring(installed, '[0-9]+\.([0-9]+)(\.|$)')::int
+        as version_from_num,
+      installed ~ 'dev|alpha|beta'
+        as version_from_isdev
+      FROM versions INTO postgis_upgrade_info
+    ;
+
+    postgis_upgrade_info_func_code := pg_catalog.format($func_code$
+        CREATE FUNCTION _postgis_upgrade_info(OUT scripts_upgraded TEXT,
+                                              OUT scripts_installed TEXT,
+                                              OUT version_to_num INT,
+                                              OUT version_from_num INT,
+                                              OUT version_from_isdev BOOLEAN)
+        AS
+        $postgis_upgrade_info$
+        BEGIN
+            scripts_upgraded := %L :: TEXT;
+            scripts_installed := %L :: TEXT;
+            version_to_num := %L :: INT;
+            version_from_num := %L :: INT;
+            version_from_isdev := %L :: BOOLEAN;
+            RETURN;
+        END
+        $postgis_upgrade_info$ LANGUAGE 'plpgsql' IMMUTABLE;
+        $func_code$,
+        postgis_upgrade_info.scripts_upgraded,
+        postgis_upgrade_info.scripts_installed,
+        postgis_upgrade_info.version_to_num,
+        postgis_upgrade_info.version_from_num,
+        postgis_upgrade_info.version_from_isdev);
+    RAISE DEBUG 'Creating function %', postgis_upgrade_info_func_code;
+    EXECUTE postgis_upgrade_info_func_code;
 END
 $$
 LANGUAGE 'plpgsql';
 
-CREATE TEMPORARY TABLE _postgis_upgrade_info AS WITH versions AS (
-  SELECT 'NEWVERSION'::text as upgraded,
-  MODULE_scripts_installed() as installed
-) SELECT
-  upgraded as scripts_upgraded,
-  installed as scripts_installed,
-  pg_catalog.substring( upgraded , '([0-9]*)\.' )::int * 100 +
-  pg_catalog.substring( upgraded , '[0-9]*\.([0-9]*)\.' )::int
-    as version_to_num,
-  pg_catalog.substring( installed , '([0-9]*)\.' )::int * 100 +
-  pg_catalog.substring( installed , '[0-9]*\.([0-9]*)\.' )::int
-    as version_from_num,
-  installed ~ 'dev|alpha|beta'
-    as version_from_isdev
-  FROM versions
-;
-
